@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -10,7 +10,7 @@ from __future__ import annotations
 
 import fsspec
 
-from .. import sync, utilities, view
+from .. import collection, fs_utils, sync, view
 
 
 def create_view(
@@ -18,6 +18,7 @@ def create_view(
     view_ref: view.ViewReference,
     *,
     filesystem: fsspec.AbstractFileSystem | str | None = None,
+    filters: collection.PartitionFilter = None,
     synchronizer: sync.Sync | None = None,
 ) -> view.View:
     """Create a new view.
@@ -26,6 +27,8 @@ def create_view(
         path: View storage directory.
         view_ref: Access properties for the reference view.
         filesystem: The file system used to access the view.
+        filters: The filters used to select the partitions of the reference
+            view. If not provided, all partitions are selected.
         synchronizer: The synchronizer used to synchronize the view.
 
     Example:
@@ -39,13 +42,14 @@ def create_view(
     Raises:
         ValueError: If the path already exists.
     """
-    filesystem = utilities.get_fs(filesystem)
+    filesystem = fs_utils.get_fs(filesystem)
     if filesystem.exists(path):
         raise ValueError(f'path {path!r} already exists.')
     return view.View(path,
                      view_ref,
                      ds=None,
                      filesystem=filesystem,
+                     filters=filters,
                      synchronizer=synchronizer)
 
 
