@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -19,6 +19,7 @@ import dask.array.routines
 import dask.array.wrap
 import dask.base
 import dask.threaded
+import fsspec
 import xarray
 
 from . import meta, variable
@@ -30,6 +31,7 @@ from .variable import (
     _attributes_repr,
     _calculate_column_width,
     _dimensions_repr,
+    _new_variable,
     _pretty_print,
 )
 
@@ -92,8 +94,8 @@ def _duplicate(var: variable.Variable,
     """
     # pylint: disable=protected-access
     # _new is a protected member of this class
-    return var._new(var.name, data, var.dimensions, var.attrs, var.compressor,
-                    var.fill_value, var.filters)
+    return _new_variable(var.name, data, var.dimensions, var.attrs,
+                         var.compressor, var.fill_value, var.filters)
     # pylint: enable=protected-access
 
 
@@ -123,12 +125,17 @@ class Dataset:
         self.dimensions: dict[str, int] = {}
 
         for var in self.variables.values():
-            for ix, dim in enumerate(var.dimensions):
-                if dim not in self.dimensions:
-                    self.dimensions[dim] = var.array.shape[ix]
-                elif self.dimensions[dim] != var.array.shape[ix]:
-                    raise ValueError(f'variable {var.name} has conflicting '
-                                     'dimensions')
+            try:
+                for ix, dim in enumerate(var.dimensions):
+                    if dim not in self.dimensions:
+                        self.dimensions[dim] = var.array.shape[ix]
+                    elif self.dimensions[dim] != var.array.shape[ix]:
+                        raise ValueError(
+                            f'variable {var.name} has conflicting '
+                            'dimensions')
+            except IndexError as exc:
+                raise ValueError(
+                    f'variable {var.name} has missing dimensions') from exc
 
     def __len__(self) -> int:
         return len(self.variables)
@@ -230,9 +237,7 @@ class Dataset:
         """
         if isinstance(names, str) or not isinstance(names, Iterable):
             names = [names]
-        # pylint: disable=expression-not-assigned
-        {self.variables.pop(name) for name in names}
-        # pylint: enable=expression-not-assigned
+        tuple(map(self.variables.pop, names))
 
     def select_vars(self, names: str | Sequence[str]) -> Dataset:
         """Return a new dataset containing only the selected variables.
@@ -386,8 +391,10 @@ class Dataset:
         arrays = tuple(item.array for item in self.variables.values())
         arrays = dask.base.compute(*arrays, **kwargs)
 
+        # Don't use _duplicate here because we because we want to transform
+        # the numpy arrays computed by dask into dask arrays
         variables = [
-            _duplicate(self.variables[k], array)
+            self.variables[k].duplicate(array)
             for k, array in zip(self.variables, arrays)
         ]
         return Dataset(variables=variables, attrs=self.attrs)
@@ -429,8 +436,9 @@ class Dataset:
                                                  fill_value=var.fill_value)
         arrays = dask.base.persist(
             *tuple(item.data for item in self.variables.values()), **kwargs)
+        variables = self.variables
         for name, array in zip(self.variables, arrays):
-            self.variables[name].array = array
+            variables[name].array = array
 
         return self
 
@@ -474,10 +482,9 @@ class Dataset:
 
         # If the dataset has common dimensions, they must be identical.
         same_dims = set(self.dimensions) & set(other.dimensions)
-        if same_dims:
-            if not all(self.dimensions[dim] == other.dimensions[dim]
-                       for dim in same_dims):
-                raise ValueError(f'dimensions {same_dims} are not identical')
+        if same_dims and not all(self.dimensions[dim] == other.dimensions[dim]
+                                 for dim in same_dims):
+            raise ValueError(f'dimensions {same_dims} are not identical')
 
         # Merge the dimensions.
         self.dimensions.update(other.dimensions)
@@ -521,6 +528,26 @@ class Dataset:
         variables = [var for var in self.variables.values() if condition(var)]
         return Dataset(variables=variables, attrs=self.attrs)
 
+    def to_zarr(self,
+                path: str,
+                fs: fsspec.AbstractFileSystem | None = None,
+                parallel: bool = True) -> None:
+        """Write the dataset to a Zarr store.
+
+        Args:
+            path: Path to the Zarr store.
+            fs: Filesystem to use.
+            parallel: If true, write the data in parallel.
+        """
+        # pylint: disable=import-outside-toplevel, import-error
+        # Avoid circular import
+        import storage
+        import sync
+
+        # pylint: enable=import-outside-toplevel, import-error
+        storage.write_zarr_group(self, path, fs or fsspec.filesystem('file'),
+                                 sync.NoSync(), parallel)
+
     def __str__(self) -> str:
         return _dataset_repr(self)
 
