@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -28,6 +28,9 @@ from . import mathematics, meta
 from .meta import Attribute
 from .type_hints import ArrayLike, NDArray, NDMaskedArray
 
+#: The dask array getter used to access the data.
+GETTER = dask.array.core.getter
+
 
 def _dimensions_repr(dimensions: dict[str, int]) -> str:
     """Get the string representation of the dimensions.
@@ -185,6 +188,89 @@ def _asarray(
     return result, fill_value
 
 
+def _new_variable(
+    name: str,
+    data: dask.array.core.Array,
+    dimensions: Sequence[str],
+    attrs: Sequence[Attribute],
+    compressor: numcodecs.abc.Codec | None,
+    fill_value: Any | None,
+    filters: Sequence[numcodecs.abc.Codec] | None,
+) -> Variable:
+    """Create a new variable.
+
+    Args:
+        name: Name of the variable
+        data: Variable data
+        dimensions: Variable dimensions
+        attrs: Variable attributes
+        compressor: Compression codec
+        fill_value: Value to use for uninitialized values
+        filters: Filters to apply before writing data to disk
+    """
+    self = Variable.__new__(Variable)
+    self.array = data
+    self.attrs = attrs
+    self.compressor = compressor
+    self.dimensions = dimensions
+    self.fill_value = fill_value
+    self.filters = filters
+    self.name = name
+    return self
+
+
+def _blockdims_from_blockshape(
+        shape: tuple[int, ...],
+        chunks: tuple[int, ...]) -> tuple[tuple[int, ...], ...]:
+    """Convert a blockshape to a blockdims tuple."""
+    return tuple(((chunk_item, ) * (shape_item // chunk_item) +
+                  ((shape_item % chunk_item, ) if shape_item %
+                   chunk_item else ()) if shape_item else (0, ))
+                 for shape_item, chunk_item in zip(shape, chunks))
+
+
+def _from_zarr_array(
+    array: zarr.Array,
+    shape: Sequence[int],
+    chunks: Sequence[int],
+    name: str,
+    lock: bool = False,
+    asarray=True,
+    inline_array=True,
+) -> dask.array.core.Array:
+    """Create a dask array from a zarr array.
+
+    Args:
+        array: A zarr array.
+
+    Returns:
+        The dask array.
+    """
+    normalized_chunks = sum(
+        (_blockdims_from_blockshape(
+            (shape_item, ),
+            (chunk_item, )) if not isinstance(chunk_item, (tuple, list)) else
+         (chunk_item, ) for shape_item, chunk_item in zip(shape, chunks)),
+        (),
+    )
+    dsk = dask.array.core.graph_from_arraylike(
+        array,
+        normalized_chunks,
+        shape,
+        name,
+        getitem=GETTER,
+        lock=lock,
+        asarray=asarray,
+        dtype=array.dtype,
+        inline_array=inline_array,
+    )
+    return dask.array.core.Array(dsk,
+                                 name,
+                                 normalized_chunks,
+                                 meta=array,
+                                 dtype=array.dtype)
+
+
 class Variable:
     """Variables hold multi-dimensional arrays of data.
 
@@ -357,37 +443,6 @@ class Variable:
                                                        self.fill_value)
         return self
 
-    @staticmethod
-    def _new(
-        name: str,
-        data: dask.array.core.Array,
-        dimensions: Sequence[str],
-        attrs: Sequence[Attribute],
-        compressor: numcodecs.abc.Codec | None,
-        fill_value: Any | None,
-        filters: Sequence[numcodecs.abc.Codec] | None,
-    ) -> Variable:
-        """Create a new variable.
-
-        Args:
-            name: Name of the variable
-            data: Variable data
-            dimensions: Variable dimensions
-            attrs: Variable attributes
-            compressor: Compression codec
-            fill_value: Value to use for uninitialized values
-            filters: Filters to apply before writing data to disk
-        """
-        self = Variable.__new__(Variable)
-        self.array = data
-        self.attrs = attrs
-        self.compressor = compressor
-        self.dimensions = dimensions
-        self.fill_value = fill_value
-        self.filters = filters
-        self.name = name
-        return self
-
     def isel(self, key: tuple[slice, ...]) -> Variable:
         """Return a new variable with data selected along the given dimension
         indices.
@@ -398,9 +453,9 @@ class Variable:
         Returns:
             The new variable
         """
-        return self._new(self.name, self.array[key], self.dimensions,
-                         self.attrs, self.compressor, self.fill_value,
-                         self.filters)
+        return _new_variable(self.name, self.array[key], self.dimensions,
+                             self.attrs, self.compressor, self.fill_value,
+                             self.filters)
 
     @classmethod
     def from_zarr(cls, array: zarr.Array, name: str, dimension: str,
@@ -420,14 +475,15 @@ class Variable:
         """
         attrs = tuple(
             Attribute(k, v) for k, v in array.attrs.items() if k != dimension)
-        data = dask.array.core.from_array(
+        data = _from_zarr_array(
             array,
+            array.shape,
             array.chunks,
             name=f'{name}-{uuid.uuid1()}',
             **kwargs,
         )
-        return cls._new(name, data, array.attrs[dimension], attrs,
-                        array.compressor, array.fill_value, array.filters)
+        return _new_variable(name, data, array.attrs[dimension], attrs,
+                             array.compressor, array.fill_value, array.filters)
 
     def duplicate(self, data: Any) -> Variable:
         """Create a new variable from the properties of this instance and the
@@ -458,8 +514,8 @@ class Variable:
         Returns:
             The variable.
         """
-        return self._new(name, self.array, self.dimensions, self.attrs,
-                         self.compressor, self.fill_value, self.filters)
+        return _new_variable(name, self.array, self.dimensions, self.attrs,
+                             self.compressor, self.fill_value, self.filters)
 
     def dimension_index(self) -> Iterator[tuple[str, int]]:
         """Return an iterator over the variable dimensions and their index.
@@ -491,10 +547,10 @@ class Variable:
             raise ValueError('other must be a non-empty sequence')
         try:
             axis = self.dimensions.index(dim)
-            return self._new(
+            return _new_variable(
                 self.name,
                 dask.array.core.concatenate(
-                    [self.array, *[item.array for item in other]], axis=axis),
+                    (self.array, *(item.array for item in other)), axis=axis),
                 self.dimensions,
                 self.attrs,
                 self.compressor,
@@ -505,9 +561,9 @@ class Variable:
             # If the concatenation dimension is not within the dimensions of the
             # variable, then the original variable is returned (i.e.
             # concatenation is not necessary).
-            return self._new(self.name, self.array, self.dimensions,
-                             self.attrs, self.compressor, self.fill_value,
-                             self.filters)
+            return _new_variable(self.name, self.array, self.dimensions,
+                                 self.attrs, self.compressor, self.fill_value,
+                                 self.filters)
 
     def rechunk(self, **kwargs) -> Variable:
         """Rechunk the variable.
@@ -519,9 +575,9 @@ class Variable:
         Returns:
             The variable.
         """
-        return self._new(self.name, self.array.rechunk(**kwargs),
-                         self.dimensions, self.attrs, self.compressor,
-                         self.fill_value, self.filters)
+        return _new_variable(self.name, self.array.rechunk(**kwargs),
+                             self.dimensions, self.attrs, self.compressor,
+                             self.fill_value, self.filters)
 
     def to_xarray(self) -> xarray.Variable:
         """Convert the variable to an xarray.Variable.
@@ -607,7 +663,9 @@ class Variable:
 
     def _dask_finalize(self, results, array_func, *args, **kwargs):
         array = array_func(results, *args, **kwargs)
-        return Variable._new(self.name, array, self.dimensions, self.attrs,
+        if not isinstance(array, dask.array.core.Array):
+            array = dask.array.core.from_array(array)
+        return _new_variable(self.name, array, self.dimensions, self.attrs,
                              self.compressor, self.fill_value, self.filters)
 
     def __dask_postcompute__(self) -> tuple:
