@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -12,6 +12,7 @@ from typing import Any, ClassVar, Iterable, Iterator
 import json
 import logging
 import pathlib
+import warnings
 
 import dask.array.core
 import dask.bag.core
@@ -19,7 +20,7 @@ import dask.distributed
 import dask.utils
 import fsspec
 
-from .. import collection, dataset, meta, storage, sync, utilities
+from .. import collection, dask_utils, dataset, fs_utils, meta, storage, sync
 from ..collection.detail import try_infer_callable
 from ..convenience import collection as convenience
 from .detail import (
@@ -27,12 +28,16 @@ from .detail import (
     _assert_have_variables,
     _assert_variable_handled,
     _create_zarr_array,
+    _deserialize_filters,
     _drop_zarr_zarr,
     _load_datasets_list,
     _load_one_dataset,
     _select_overlap,
+    _serialize_filters,
+    _sync,
     _wrap_update_func,
     _wrap_update_func_overlap,
+    _write_checksum,
 )
 
 __all__ = ['View', 'ViewReference']
@@ -49,6 +54,8 @@ class View:
         view_ref: Access properties for the reference view.
         ds: The dataset handled by this view.
         filesystem: The file system used to access the view.
+        filters: The filters used to select the partitions of the reference. If
+            not provided, all partitions are selected.
         synchronizer: The synchronizer used to synchronize the view.
 
     .. note::
@@ -67,12 +74,13 @@ class View:
         *,
         ds: meta.Dataset | None,
         filesystem: fsspec.AbstractFileSystem | str | None = None,
+        filters: collection.PartitionFilter = None,
         synchronizer: sync.Sync | None = None,
     ) -> None:
         #: The file system used to access the view (default local file system).
-        self.fs = utilities.get_fs(filesystem)
+        self.fs = fs_utils.get_fs(filesystem)
         #: Path to the directory where the view is stored.
-        self.base_dir = utilities.normalize_path(self.fs, base_dir)
+        self.base_dir = fs_utils.normalize_path(self.fs, base_dir)
         #: The reference collection of the view.
         self.view_ref = convenience.open_collection(
             view_ref.path, mode='r', filesystem=view_ref.filesystem)
@@ -81,11 +89,39 @@ class View:
             self.view_ref.metadata.dimensions, variables=[], attrs=[])
         #: The synchronizer used to synchronize the view.
         self.synchronizer = synchronizer or sync.NoSync()
+        #: The filters used to select the partitions of the reference.
+        self.filters = filters
 
         if not self.fs.exists(self.base_dir):
             _LOGGER.info('Creating view %s', self)
             self.fs.makedirs(self.base_dir)
             self._write_config()
+            self._init_partitions(filters)
+        else:
+            _LOGGER.info('Opening view %s', self)
+
+    def _init_partitions(self, filters: collection.PartitionFilter) -> None:
+        """Initialize the partitions of the view."""
+        _LOGGER.info('Populating view %s', self)
+        args = tuple(
+            map(lambda item: fs_utils.join_path(self.base_dir, item),
+                self.view_ref.partitions(filters=filters, relative=True)))
+        # When opening an existing view, if the user asks to use new partitions
+        # from the reference collection, in this case only the missing
+        # partitions are created.
+        args = tuple(filter(lambda item: not self.fs.exists(item), args))
+        _LOGGER.info('%d partitions selected from %s', len(args),
+                     self.view_ref)
+        client = dask_utils.get_client()
+        storage.execute_transaction(
+            client, self.synchronizer,
+            client.map(
+                _write_checksum,
+                tuple(args),
+                base_dir=self.base_dir,
+                view_ref=self.view_ref,
+                fs=self.fs,
+            ))
 
     def __str__(self) -> str:
         return (f'{self.__class__.__name__}'
@@ -93,20 +129,25 @@ class View:
                 f'base_dir={self.base_dir!r}>')
 
     @classmethod
-    def _config(cls, base_dir: str, fs: fsspec.AbstractFileSystem) -> str:
+    def _config(cls, base_dir: str) -> str:
         """Returns the configuration path."""
-        return fs.sep.join((base_dir, cls.CONFIG))
+        return fs_utils.join_path(base_dir, cls.CONFIG)
 
     def _write_config(self) -> None:
         """Write the configuration file for the view."""
-        config = self._config(self.base_dir, self.fs)
+        config = self._config(self.base_dir)
         fs = json.loads(self.view_ref.fs.to_json())
         with self.fs.open(config, mode='w') as stream:
             json.dump(
-                dict(base_dir=self.base_dir,
-                     metadata=self.metadata.get_config(),
-                     view_ref=dict(path=self.view_ref.partition_properties.dir,
-                                   fs=fs)),
+                {
+                    'base_dir': self.base_dir,
+                    'filters': _serialize_filters(self.filters),
+                    'metadata': self.metadata.get_config(),
+                    'view_ref': {
+                        'path': self.view_ref.partition_properties.dir,
+                        'fs': fs,
+                    },
+                },
                 stream,  # type: ignore[arg-type]
                 indent=4)
         self.fs.invalidate_cache(config)
@@ -133,8 +174,8 @@ class View:
             ValueError: If the provided directory does not contain a view.
         """
         _LOGGER.info('Opening view %r', path)
-        fs = utilities.get_fs(filesystem)
-        config = cls._config(path, fs)
+        fs = fs_utils.get_fs(filesystem)
+        config = cls._config(path)
         if not fs.exists(config):
             raise ValueError(f'zarr view not found at path {path!r}')
         with fs.open(config) as stream:
@@ -148,6 +189,7 @@ class View:
                             json.dumps(view_ref['fs']))),
                     ds=meta.Dataset.from_config(data['metadata']),
                     filesystem=filesystem,
+                    filters=_deserialize_filters(data['filters']),
                     synchronizer=synchronizer)
 
     def partitions(
@@ -164,7 +206,7 @@ class View:
         """
         return filter(
             self.fs.exists,
-            map(lambda item: self.fs.sep.join((self.base_dir, item)),
+            map(lambda item: fs_utils.join_path(self.base_dir, item),
                 self.view_ref.partitions(filters=filters, relative=True)))
 
     def variables(
@@ -211,24 +253,21 @@ class View:
         if (variable.name in self.view_ref.metadata.variables
                 or variable.name in self.metadata.variables):
             raise ValueError(f'Variable {variable.name} already exists')
-        client = utilities.get_client()
+        client = dask_utils.get_client()
         self.metadata.add_variable(variable)
         template = self.view_ref.metadata.search_same_dimensions_as(variable)
 
-        existing_partitions: Iterable[str] = tuple(self.partitions())
+        existing_partitions = {
+            pathlib.Path(path).relative_to(self.base_dir).as_posix()
+            for path in self.partitions()
+        }
 
-        # If the view already contains variables, you only need to modify the
-        # existing partitions.
-        if existing_partitions:
-            existing_partitions = {
-                pathlib.Path(path).relative_to(self.base_dir).as_posix()
-                for path in existing_partitions
-            }
-            args: Any = filter(lambda item: item[0] in existing_partitions,
-                               self.view_ref.iterate_on_records(relative=True))
-        else:
-            args = self.view_ref.iterate_on_records(relative=True)
+        if len(existing_partitions) == 0:
+            _LOGGER.info('No partitions found, skipping variable creation')
+            return
 
+        args = filter(lambda item: item[0] in existing_partitions,
+                      self.view_ref.iterate_on_records(relative=True))
         try:
             storage.execute_transaction(
                 client, self.synchronizer,
@@ -270,7 +309,7 @@ class View:
         _LOGGER.info('Dropping variable %r', varname)
         _assert_variable_handled(self.view_ref.metadata, self.metadata,
                                  varname)
-        client = utilities.get_client()
+        client = dask_utils.get_client()
 
         variable = self.metadata.variables.pop(varname)
         self._write_config()
@@ -321,7 +360,7 @@ class View:
             arguments = tuple((self.view_ref.partitioning.parse(item), [])
                               for item in self.partitions(filters=filters))
 
-        client = utilities.get_client()
+        client = dask_utils.get_client()
         futures = client.map(
             _load_one_dataset,
             arguments,
@@ -399,13 +438,20 @@ class View:
             >>> view.update(update_temperature)
         """
         _assert_have_variables(self.metadata)
-        client = utilities.get_client()
+
+        client = dask_utils.get_client()
 
         datasets_list = tuple(
             _load_datasets_list(client, self.base_dir, self.fs,
                                 self.view_ref, self.metadata,
                                 self.partitions(filters), selected_variables))
 
+        # If no dataset is selected, we have nothing to do.
+        if not datasets_list:
+            warnings.warn('The update function is not applied because no '
+                          'data is selected with the given filters.')
+            return
+
         func_result = try_infer_callable(
             func, datasets_list[0][0], self.view_ref.partition_properties.dim,
             *args, **kwargs)
@@ -436,9 +482,9 @@ class View:
                 **kwargs,
             )
 
-        batchs = utilities.split_sequence(
+        batchs = dask_utils.split_sequence(
             datasets_list, partition_size
-            or utilities.dask_workers(client, cores_only=True))
+            or dask_utils.dask_workers(client, cores_only=True))
         awaitables = client.map(wrap_function,
                                 tuple(batchs),
                                 key=func.__name__,
@@ -509,7 +555,7 @@ class View:
 
         _assert_have_variables(self.metadata)
 
-        client = utilities.get_client()
+        client = dask_utils.get_client()
         datasets_list = tuple(
             _load_datasets_list(client, self.base_dir, self.fs,
                                 self.view_ref, self.metadata,
@@ -566,6 +612,9 @@ class View:
             ...     print(item)
             [1.0, 2.0, 3.0, 4.0]
         """
+        if depth < 0:
+            raise ValueError('Depth must be greater than or equal to 0')
+
         add_partition_info = dask.utils.has_keyword(func, 'partition_info')
 
         def _wrap(
@@ -594,7 +643,8 @@ class View:
 
             if add_partition_info:
                 kwargs = kwargs.copy()
-                kwargs['partition_info'] = indices
+                kwargs['partition_info'] = (
+                    self.view_ref.partition_properties.dim, indices)
 
             # Finally, apply the function.
             return (self.view_ref.partitioning.parse(arguments[1]),
@@ -602,7 +652,7 @@ class View:
 
         _assert_have_variables(self.metadata)
 
-        client = utilities.get_client()
+        client = dask_utils.get_client()
         datasets_list = tuple(
             _load_datasets_list(client, self.base_dir, self.fs,
                                 self.view_ref, self.metadata,
@@ -611,3 +661,76 @@ class View:
                                           partition_size=partition_size,
                                           npartitions=npartitions)
         return bag.map(_wrap, func, datasets_list, depth, *args, **kwargs)
+
+    def is_synced(self) -> bool:
+        """Check if the view is synchronized with the underlying collection.
+
+        Returns:
+            True if the view is synchronized, False otherwise.
+        """
+        partitions = tuple(self.view_ref.partitions(relative=True))
+        client = dask_utils.get_client()
+        unsynchronized_partition = storage.execute_transaction(
+            client, self.synchronizer,
+            client.map(_sync,
+                       partitions,
+                       base_dir=self.base_dir,
+                       fs=self.fs,
+                       view_ref=self.view_ref,
+                       metadata=self.metadata,
+                       dry_run=True))
+        return len(
+            tuple(
+                filter(lambda item: item is not None,
+                       unsynchronized_partition))) == 0
+
+    def sync(
+        self,
+        filters: collection.PartitionFilter = None
+    ) -> collection.PartitionFilterCallback:
+        """Synchronize the view with the underlying collection.
+
+        This method is useful to update the view after a change in the
+        underlying collection.
+
+        Args:
+            filters: The predicate used to select the partitions to
+                synchronize. To get more information on the predicate, see the
+                documentation of the :meth:`zcollection.Collection.partitions`
+                method.
+                If None, the view is synchronized with all the partitions
+                already present in the view. If you want to extend the view
+                with new partitions, use must provide a predicate that
+                selects the new partitions.
+                Existing partitions are not removed, even if they are not
+                selected by the predicate.
+
+        Returns:
+            A function that can be used as a predicate to get the partitions
+            that have been synchronized using the :meth:`View.partitions`
+            method.
+        """
+        _LOGGER.info('Synchronizing view %s', self)
+
+        if filters is not None:
+            self.filters = filters
+            self._write_config()
+            self._init_partitions(filters)
+
+        partitions = tuple(self.view_ref.partitions(relative=True))
+        _LOGGER.info('%d partitions to synchronize', len(partitions))
+
+        client = dask_utils.get_client()
+        synchronized_partition = storage.execute_transaction(
+            client, self.synchronizer,
+            client.map(_sync,
+                       partitions,
+                       base_dir=self.base_dir,
+                       fs=self.fs,
+                       view_ref=self.view_ref,
+                       metadata=self.metadata))
+        partition_ids = []
+        for item in filter(lambda item: item is not None,
+                           synchronized_partition):
+            partition_ids.append(dict(self.view_ref.partitioning.parse(item)))
+        return lambda item: item in partition_ids
